package org.gcube.portlets.user.td.codelistmappingimportwidget.client;

import org.gcube.portlets.user.td.gwtservice.client.rpc.TDGWTServiceAsync;
import org.gcube.portlets.user.td.gwtservice.shared.codelisthelper.CodelistMappingSession;
import org.gcube.portlets.user.td.gwtservice.shared.exception.TDGWTSessionExpiredException;
import org.gcube.portlets.user.td.gwtservice.shared.tr.TabResource;
import org.gcube.portlets.user.td.gwtservice.shared.tr.TableData;
import org.gcube.portlets.user.td.widgetcommonevent.client.event.SessionExpiredEvent;
import org.gcube.portlets.user.td.widgetcommonevent.client.type.SessionExpiredType;
import org.gcube.portlets.user.td.wizardwidget.client.WizardCard;

import com.allen_sauer.gwt.log.client.Log;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.sencha.gxt.widget.core.client.box.AlertMessageBox;
import com.sencha.gxt.widget.core.client.box.ConfirmMessageBox;
import com.sencha.gxt.widget.core.client.event.DialogHideEvent;
import com.sencha.gxt.widget.core.client.event.DialogHideEvent.DialogHideHandler;
//Next in GXT 3.1.1
//import com.sencha.gxt.widget.core.client.event.DialogHideEvent;
//import com.sencha.gxt.widget.core.client.event.DialogHideEvent.DialogHideHandler;
import com.sencha.gxt.widget.core.client.event.HideEvent;
import com.sencha.gxt.widget.core.client.event.HideEvent.HideHandler;

public class TabResourcesSelectionCard extends WizardCard {
	protected CodelistMappingSession codelistMappingSession;
	protected TabResourcesSelectionCard thisCard;
	protected TabResourcesSelectionPanel tabResourcesSelectionPanel;
	protected TabResource selectedTabResource = null;

	public TabResourcesSelectionCard(
			final CodelistMappingSession codelistMappingSession) {
		super("Select the Codelist before curation", "");
		Log.debug("TabResourcesSelectionCard");
		this.codelistMappingSession = codelistMappingSession;
		thisCard = this;

		tabResourcesSelectionPanel = new TabResourcesSelectionPanel(thisCard,
				res);

		tabResourcesSelectionPanel
				.addSelectionHandler(new SelectionHandler<TabResource>() {

					public void onSelection(SelectionEvent<TabResource> event) {
						codelistMappingSession
								.setConnectedTR(tabResourcesSelectionPanel
										.getSelectedItem());
						getWizardWindow().setEnableNextButton(true);
					}

				});

		setContent(tabResourcesSelectionPanel);

	}

	@Override
	public void setup() {
		Log.debug("TabResourcesSelectionCard Call Setup ");
		Command sayNextCard = new Command() {

			public void execute() {
				Log.debug("TabResourcesSelectionCard Call sayNextCard");
				retrieveLastTable();

			}

		};

		getWizardWindow().setNextButtonCommand(sayNextCard);

		Command sayPreviousCard = new Command() {
			public void execute() {
				try {
					getWizardWindow().previousCard();
					getWizardWindow().removeCard(thisCard);
					Log.debug("Remove TabResourcesSelectionCard");
				} catch (Exception e) {
					Log.error("sayPreviousCard :" + e.getLocalizedMessage());
				}
			}
		};

		getWizardWindow().setPreviousButtonCommand(sayPreviousCard);
		getWizardWindow().setEnableNextButton(false);
		getWizardWindow().setEnableBackButton(true);

	}

	protected void retrieveLastTable() {
		getWizardWindow().setEnableNextButton(false);
		getWizardWindow().setEnableBackButton(false);

		TDGWTServiceAsync.INSTANCE.getLastTable(codelistMappingSession
				.getConnectedTR().getTrId(), new AsyncCallback<TableData>() {

			@Override
			public void onFailure(Throwable caught) {
				if (caught instanceof TDGWTSessionExpiredException) {
					getEventBus().fireEvent(
							new SessionExpiredEvent(
									SessionExpiredType.EXPIREDONSERVER));
				} else {
					Log.debug("Attention",
							"This tabular resource does not have a valid table");
					AlertMessageBox d = new AlertMessageBox("Attention",
							"This tabular resource does not have a valid table");
					d.addHideHandler(new HideHandler() {

						public void onHide(HideEvent event) {
							deleteTRWithLastTableNull();

						}
					});
					d.show();

				}
			}

			@Override
			public void onSuccess(TableData result) {
				Log.debug("Retrieve last table: " + result);
				updateConnectedTRInfo(result);
			}

		});

	}

	protected void deleteTRWithLastTableNull() {
		final ConfirmMessageBox mb = new ConfirmMessageBox("Delete",
				"Would you like to delete this tabular resource without table?");
		/* GXT 3.0.1
		mb.addHideHandler(new HideHandler() {
			public void onHide(HideEvent event) {
				if (mb.getHideButton() == mb.getButtonById(PredefinedButton.YES
						.name())) {
					callDeleteLastTable();
				} else if (mb.getHideButton() == mb
						.getButtonById(PredefinedButton.NO.name())) {
					getWizardWindow().setEnableNextButton(true);
					getWizardWindow().setEnableBackButton(true);
				}
			}
		});
		*/
		
		// Next in GXT 3.1.1
		 
		  mb.addDialogHideHandler(new DialogHideHandler() {
		 

			@Override
			public void onDialogHide(DialogHideEvent event) {
				switch (event.getHideButton()) {
				case NO:
					getWizardWindow().setEnableNextButton(true);
					getWizardWindow().setEnableBackButton(true);
					break;
				case YES:
					callDeleteLastTable();
					break;
				default:
					getWizardWindow().setEnableNextButton(true);
					getWizardWindow().setEnableBackButton(true);
					break;

				}

			}
		});
		mb.setWidth(300);
		mb.show();

	}

	protected void callDeleteLastTable() {
		Log.debug("Delete TR:"
				+ codelistMappingSession.getConnectedTR().getTrId());
		TDGWTServiceAsync.INSTANCE.removeTabularResource(codelistMappingSession
				.getConnectedTR().getTrId(), new AsyncCallback<Void>() {

			public void onFailure(Throwable caught) {
				AlertMessageBox d = new AlertMessageBox("Error",
						"Error on delete TabResource: "
								+ caught.getLocalizedMessage());
				d.addHideHandler(new HideHandler() {

					public void onHide(HideEvent event) {
						getWizardWindow().setEnableNextButton(true);
						getWizardWindow().setEnableBackButton(true);

					}
				});
				d.show();

			}

			public void onSuccess(Void result) {
				tabResourcesSelectionPanel.gridReload();
				getWizardWindow().setEnableNextButton(false);
				getWizardWindow().setEnableBackButton(true);
			}

		});
	}

	protected void updateConnectedTRInfo(TableData table) {
		TabResource tabResource = codelistMappingSession.getConnectedTR();
		tabResource.setTrId(table.getTrId());
		codelistMappingSession.setConnectedTR(tabResource);
		Log.debug("CodelistMappingSession: " + codelistMappingSession);
		goNext();
	}

	protected void goNext() {
		try {
			Log.info("NextCard ColumnSelectionCard");
			ColumnSelectionCard columnSelectionCard = new ColumnSelectionCard(
					codelistMappingSession);
			getWizardWindow().addCard(columnSelectionCard);
			getWizardWindow().nextCard();
		} catch (Throwable e) {
			Log.error("goNext: " + e.getLocalizedMessage());
		}
	}

}
